use application::util::websocket::{server::AppState, server::WebSocketState};

#[cfg(feature = "ssr")]
#[tokio::main]
async fn main() {
    use application::fileserv::file_and_error_handler;
    use application::{app::*, util::websocket::server};
    use axum::{routing::get, Router};
    use leptos::*;
    use leptos_axum::{generate_route_list, LeptosRoutes};
    use std::collections::HashSet;
    use std::sync::Arc;
    use tokio::sync::{broadcast, Mutex};

    application::util::surrealdb::connect()
        .await
        .expect("Database connection failed");

    // Setting get_configuration(None) means we'll be using cargo-leptos's env values
    // For deployment these variables are:
    // <https://github.com/leptos-rs/start-axum#executing-a-server-on-a-remote-machine-without-the-toolchain>
    // Alternately a file can be specified such as Some("Cargo.toml")
    // The file would need to be included with the executable when moved to deployment
    let conf = get_configuration(None).await.unwrap();
    let leptos_options = conf.leptos_options;
    let addr = leptos_options.site_addr;
    let routes = generate_route_list(App);

    let client_set = Arc::new(Mutex::new(HashSet::<uuid::Uuid>::new()));
    let (tx, _rx) = broadcast::channel(100);

    let websocket_state = WebSocketState { client_set, tx };

    let app_state = AppState {
        websocket_state: websocket_state.into(),
        routes: routes.clone(),
        leptos_options,
    };

    // build our application with a route
    let app = Router::new()
        .route("/ws", get(server::websocket_handler))
        .leptos_routes(&app_state, routes, App)
        .fallback(file_and_error_handler)
        .with_state(app_state);

    let listener = tokio::net::TcpListener::bind(&addr).await.unwrap();
    logging::log!("listening on http://{}", &addr);

    axum::serve(listener, app.into_make_service())
        .await
        .unwrap();
}

#[cfg(not(feature = "ssr"))]
pub fn main() {
    // no client-side main function
    // unless we want this to work with e.g., Trunk for a purely client-side app
    // see lib.rs for hydration function instead
}
